@@ -132,21 +132,39 @@ class ConfuseCallback(tf.keras.callbacks.Callback):
 
 
 class ModelSaverCallback(tf.keras.callbacks.Callback):
-    def __init__(self, best_loss=float('inf'), save_root="./", enable=True, epoch=0):
+    def __init__(self, best_metric=float('inf'), save_root="./", save_metric='val_loss', enable=True, epoch=0):
+        """
+
+        :param best_metric: Set best score of previous training session if resuming
+        :param save_root: Model save path
+        :param save_metric: One of 'val_loss', 'val_accuracy'
+        :param enable: (bool)
+        :param epoch: Set previous epoch number if resuming
+        """
         super(ModelSaverCallback, self).__init__()
-        self.best_loss = best_loss
+        self.best_metric = best_metric
+        if self.best_metric == float('inf') and save_metric.find("accuracy") > 0:
+            self.best_metric = -self.best_metric
+
         self.epoch = epoch
         self.save_root = save_root
         self.enable = enable
+        self.save_metric = save_metric
 
     def on_epoch_end(self, epoch, logs=None):
         try:
             epoch += self.epoch
-            if logs['val_loss'] < self.best_loss:
+            a = logs[self.save_metric]
+            b = self.best_metric
+
+            if self.save_metric.find("accuracy") > 0:
+                a, b = b, a
+
+            if a < b:
                 # TODO Delete previous saved model if exists
-                file_name = '{}my_model_weight_{:04d}_{:03.2f}.h5'.format(self.save_root, epoch, logs['val_loss'])
-                print("\nBest loss! saving the model to {} ...".format(file_name))
-                self.best_loss = logs['val_loss']
+                file_name = '{}my_model_weight_{:04d}_{}_{:03.2f}.h5'.format(self.save_root, epoch, self.save_metric, logs[self.save_metric])
+                print("\nBest score! saving the model to {} ...".format(file_name))
+                self.best_metric = logs[self.save_metric]
 
                 if self.enable:
                     self.model.save(file_name)
@@ -176,7 +194,7 @@ def wait_ctrl_c(pre_msg="Press Ctrl+c to quit Tensorboard", post_msg="\nExit."):
 def get_tf_callbacks(root,
                      tboard_callback=True, tboard_update_freq='epoch', tboard_histogram_freq=1, tboard_profile_batch=0,
                      confuse_callback=True, label_info=None, x_test=None, y_test=None, test_generator_=None, test_dataset=None, figure_size=(12, 10),
-                     modelsaver_callback=False, best_loss=float('inf'), save_root=None, best_epoch=0,
+                     modelsaver_callback=False, best_loss=float('inf'), save_root=None, best_epoch=0, save_metric='val_loss',
                      earlystop_callback=True, earlystop_monitor='val_loss', earlystop_patience=0, earlystop_restore_weights=True):
     """
     Get TensorFlow callbacks function for convinience
@@ -201,6 +219,7 @@ def get_tf_callbacks(root,
     :param best_loss:
     :param save_root:
     :param best_epoch:
+    :param save_metric:
     :param earlystop_callback:
     :param earlystop_monitor:
     :param earlystop_patience:
@@ -238,7 +257,7 @@ def get_tf_callbacks(root,
         if not save_root:
             save_root = log_root_
         callbacks_.append(
-            ModelSaverCallback(best_loss=best_loss, save_root=save_root, epoch=best_epoch)
+            ModelSaverCallback(best_metric=best_loss, save_root=save_root, epoch=best_epoch, save_metric=save_metric)
         )
 
     if earlystop_callback:
